# -*- coding: utf-8 -*-
"""Action script for push.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Push all repository to the upper server"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(_my_args, _section):
    _my_args.add("r", "remote", haveParam=True, desc="Name of the remote server")


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_remote_name = ""
    for elem in _arguments:
        if elem.get_option_name() == "remote":
            debug.info("find remote name: '" + elem.get_arg() + "'")
            argument_remote_name = elem.get_arg()
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")
    mani = manifest.Manifest(file_source_manifest)

    all_project = mani.get_all_configs()
    debug.info("fetch : " + str(len(all_project)) + " projects")
    id_element = 0
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.info("push: " + base_display)
        tools.wait_for_server_if_needed()
        # debug.debug("elem : " + str(elem))
        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        if os.path.exists(git_repo_path) is False:
            debug.error("can not push project that not exist")
            continue

        if os.path.exists(os.path.join(git_repo_path, ".git")) is False:
            # path already exist but it is not used to as a git repo ==> this is an error
            debug.error("path '" + git_repo_path + "' exist but not used for a git repository. Clean it and restart")

        # get the current branch:
        # get local branch
        cmd = "git branch -a"
        debug.verbose("execute : " + cmd)
        ret_branch = multiprocess.run_command(cmd, cwd=git_repo_path)
        list_branch = ret_branch[1].split("\n")
        list_branch2 = []
        list_branch3 = []
        select_branch = ""
        for elem_branch in list_branch:
            if len(elem_branch.split(" -> ")) != 1:
                continue
            if elem_branch[2:10] == "remotes/":
                elem_branch = elem_branch[:2] + elem_branch[10:]
            if elem_branch[:2] == "* ":
                list_branch2.append([elem_branch[2:], True])
                select_branch = elem_branch[2:]
            else:
                list_branch2.append([elem_branch[2:], False])
            list_branch3.append(elem_branch[2:])

        # simply update the repository ...
        debug.verbose("Push project: ")
        # fetch the repository
        cmd = "git push"
        if argument_remote_name != "":
            cmd += " " + argument_remote_name
        else:
            cmd += " " + elem.select_remote["name"]
        cmd += " " + select_branch + ":" + select_branch
        debug.info("execute : " + cmd)
        multiprocess.run_command_direct(cmd, cwd=git_repo_path)



# -*- coding: utf-8 -*-
"""Action script for status.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import status

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Get the status of all the repositories"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(_my_args, _section):
    _my_args.add("r", "remote", haveParam=True, desc="Name of the remote server")
    _my_args.add(
        "t",
        "tags",
        haveParam=False,
        desc="Display if the commit is on a tag (and display it)",
    )


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_remote_name = ""
    argument_display_tag = False
    for elem in _arguments:
        if elem.get_option_name() == "remote":
            debug.info("find remote name: '" + elem.get_arg() + "'")
            argument_remote_name = elem.get_arg()
        elif elem.get_option_name() == "tags":
            argument_display_tag = True
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()
    configuration = config.get_unique_config()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")

    mani = manifest.Manifest(file_source_manifest)
    is_modify_manifest = commands.check_repository_is_modify(env.get_island_path_manifest())
    if is_modify_manifest is True:
        debug.info("!!!!!!!!!!!! MANIFEST is modify !!!!!!!!")

    all_project = mani.get_all_configs()
    debug.info("status of: " + str(len(all_project)) + " projects")
    id_element = 0

    elem = configuration.get_manifest_config()
    base_display = tools.get_list_base_display(id_element, len(all_project), elem)
    status.display_status(elem, argument_remote_name, argument_display_tag, id_element, base_display)

    is_behind = False
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        ret = status.display_status(elem, argument_remote_name, argument_display_tag, id_element, base_display)
        if ret != None:
            is_behind = True

    if is_behind is True:
        return env.ret_action_need_update



# -*- coding: utf-8 -*-
"""Status interface display.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


default_behind_message = "[DEV] update dev tag version"
default_update_message = "[VERSION] update dev tag version"


base_name_of_a_tagged_branch = "branch_on_tag_"


def display_status(elem, argument_remote_name, argument_display_tag, id_element, base_display):
    volatile = ""
    if elem.volatile is True:
        volatile = " (volatile)"
    debug.verbose("status : " + base_display)
    # debug.debug("elem : " + str(elem))
    git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
    if os.path.exists(git_repo_path) is False:
        debug.info(base_display + volatile + "\r\t\t\t\t\t\t\t\t\t" + "     (not download)")
        return

    is_modify = commands.check_repository_is_modify(git_repo_path)
    list_branch = commands.get_list_branch_all(git_repo_path)
    select_branch = commands.get_current_branch(git_repo_path)
    debug.verbose("List all branch: " + str(list_branch))
    if select_branch[: len(base_name_of_a_tagged_branch)] != base_name_of_a_tagged_branch:
        # get tracking branch
        tracking_remote_branch = commands.get_tracking_branch(git_repo_path, argument_remote_name, select_branch)
        if tracking_remote_branch == None:
            debug.info(base_display + volatile + "\r\t\t\t\t\t\t\t     (NO BRANCH)")
            return
    else:
        tracking_remote_branch = select_branch[len(base_name_of_a_tagged_branch) :]
    modify_status = "     "
    if is_modify is True:
        modify_status = " *** "

    debug.verbose("select branch = '" + select_branch + "' is modify : " + str(is_modify) + "     track: '" + str(tracking_remote_branch) + "'")

    ret_current_branch_sha1 = commands.get_revision_list_to_branch(git_repo_path, select_branch)
    ret_track_branch_sha1 = commands.get_revision_list_to_branch(git_repo_path, tracking_remote_branch)
    # remove all identical sha1 ==> not needed for this
    in_forward = 0
    for elem_sha1 in ret_current_branch_sha1:
        if elem_sha1 not in ret_track_branch_sha1:
            in_forward += 1
    in_behind = 0
    for elem_sha1 in ret_track_branch_sha1:
        if elem_sha1 not in ret_current_branch_sha1:
            in_behind += 1

    behind_forward_comment = ""
    if in_forward != 0:
        behind_forward_comment += "forward=" + str(in_forward)
    if in_behind != 0:
        if in_forward != 0:
            behind_forward_comment += " "
        behind_forward_comment += "behind=" + str(in_behind)
    if behind_forward_comment != "":
        behind_forward_comment = "\r\t\t\t\t\t\t\t\t\t\t\t\t[" + behind_forward_comment + "]"

    tags_comment = ""
    # check the current tags of the repository
    if argument_display_tag is True:
        ret_current_tags = commands.get_tags_current(git_repo_path)
        debug.verbose("tags found: " + str(ret_current_tags))
        for elem_tag in ret_current_tags:
            if len(tags_comment) != 0:
                tags_comment += ","
            tags_comment += elem_tag
        if len(tags_comment) != 0:
            tags_comment = "\r\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t[" + tags_comment + "]"
        else:
            tags_comment = "\r\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t- - - - -"
    debug.info(
        base_display
        + volatile
        + "\r\t\t\t\t\t\t\t"
        + modify_status
        + "("
        + select_branch
        + " -> "
        + tracking_remote_branch
        + ")"
        + behind_forward_comment
        + tags_comment
    )
    if is_modify is True:
        cmd = "git status --short"
        debug.verbose("execute : " + cmd)
        ret_diff = multiprocess.run_command(cmd, cwd=git_repo_path)
        tmp_color_red = "\033[31m"
        tmp_color_default = "\033[00m"
        debug.info(tmp_color_red + ret_diff[1] + tmp_color_default)
    return in_behind


def deliver_check(
    elem,
    argument_remote_name,
    id_element,
    base_display,
    source_branch,
    destination_branch,
):
    deliver_available = True
    debug.debug("deliver-ckeck: " + base_display)
    debug.debug("    ==> check repo exist")
    # Check the repo exist
    git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
    if os.path.exists(git_repo_path) is False:
        debug.warning("deliver-ckeck: " + base_display + " ==> MUST be download")
        return False
    debug.debug("    ==> check is modify")
    # check if the curent repo is modify
    is_modify = commands.check_repository_is_modify(git_repo_path)
    if is_modify is True:
        debug.warning("deliver-ckeck: " + base_display + " ==> MUST not be modify")
        return False

    debug.debug("    ==> check current branch is '" + source_branch + "'")
    # check if we are on source_branch
    select_branch = commands.get_current_branch(git_repo_path)
    if select_branch != source_branch:
        debug.warning("deliver-ckeck: " + base_display + " ==> MUST be on source branch: '" + source_branch + "' and is: '" + select_branch + "'")
        return False
    debug.debug("    ==> check have tracking branch")
    # check if we have a remote traking branch
    tracking_remote_branch = commands.get_tracking_branch(git_repo_path, argument_remote_name, select_branch)
    if tracking_remote_branch == None:
        debug.warning("deliver-ckeck: " + base_display + " ==> MUST have a remote tracking branch")
        deliver_available = False

    # go on destination branch
    commands.checkout(git_repo_path, destination_branch)
    # TODO: check return value

    debug.debug("    ==> check current branch is '" + source_branch + "'")
    # check if we are on "master"
    select_branch = commands.get_current_branch(git_repo_path)
    if select_branch != destination_branch:
        debug.warning("deliver-ckeck: " + base_display + " ==> Can not checkout branch: '" + destination_branch + "' and is: '" + select_branch + "'")
        deliver_available = False
    debug.debug("    ==> check have tracking branch")
    # check if we have a remote traking branch
    tracking_remote_branch = commands.get_tracking_branch(git_repo_path, argument_remote_name, select_branch)
    if tracking_remote_branch == None:
        debug.warning("deliver-ckeck: " + base_display + " ==> MUST have a remote tracking branch")
        deliver_available = False

    """
	# check if we have a local branch
	list_branch_local = commands.get_list_branch_local(git_repo_path)
	if destination_branch not in list_branch_local:
		debug.warning("deliver-ckeck: " + base_display + " ==> MUST have local branch named '" + destination_branch + "'")
		deliver_available = False
	# TODO: check source_branch is up to date

	# TODO: check the remote branch and the local branch are the same
	#sha_tracking = get_sha1_for_branch(git_repo_path, tracking_remote_branch)
	#sha_current = get_sha1_for_branch(git_repo_path, select_branch)
	"""

    # check out back the source branch
    commands.checkout(git_repo_path, source_branch)
    return deliver_available


def checkout_elem(elem, argument_remote_name, branch_to_checkout, base_display):
    debug.verbose("checkout : " + base_display)
    git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
    if os.path.exists(git_repo_path) is False:
        debug.warning("checkout " + base_display + " ==> repository does not exist ...")
        return False

    # check if the repository is modify
    is_modify = commands.check_repository_is_modify(git_repo_path)
    if is_modify is True:
        debug.warning("checkout " + base_display + " ==> modify data can not checkout new branch")
        return False

    list_branch_local = commands.get_list_branch_local(git_repo_path)
    select_branch = commands.get_current_branch(git_repo_path)

    is_tag = False
    if branch_to_checkout == "__TAG__":
        branch_to_checkout = base_name_of_a_tagged_branch + str(elem.tag)
        is_tag = True
        if elem.volatile is True:
            debug.info("checkout " + base_display + " ==> Can not checkout for 'volatile' repository")
            return True
        if elem.tag == None:
            debug.info("checkout " + base_display + " ==> Can not checkout for '''None''' Tag")
            return True
    # check if we are on the good branch:
    if branch_to_checkout == select_branch:
        debug.info("checkout " + base_display + " ==> No change already on good branch")
        return True

    # check if we have already checkout the branch before
    debug.verbose("      check : " + branch_to_checkout + "    in " + str(list_branch_local))
    if branch_to_checkout in list_branch_local:
        cmd = "git checkout " + branch_to_checkout
        debug.verbose("execute : " + cmd)
        ret = multiprocess.run_command(cmd, cwd=git_repo_path)
        if ret[0] != 0 and ret[1] != "" and ret != False:
            debug.info("'" + str(ret) + "'")
            debug.error("checkout " + base_display + " ==> Can not checkout to the correct branch")
            return False
        debug.info("checkout " + base_display + " ==> switch branch")
        # TODO : Check the number of commit to the origin/XXX branch ....
        return True

    list_tags = commands.get_tags(git_repo_path)
    if branch_to_checkout in list_tags:
        is_tag = True
        if elem.tag == None:
            elem.tag = branch_to_checkout
            branch_to_checkout = base_name_of_a_tagged_branch + str(elem.tag)

    # Check if the remote branch exist ...
    if is_tag is False:
        list_branch_remote = commands.get_list_branch_remote(git_repo_path)
        if elem.select_remote["name"] + "/" + branch_to_checkout in list_branch_remote:
            debug.info("    ==> find ...")
        else:
            debug.info("checkout " + base_display + " ==> NO remote branch")
            return True
        # checkout the new branch:
        cmd = "git checkout --quiet " + elem.select_remote["name"] + "/" + branch_to_checkout + " -b " + branch_to_checkout
        # + " --track " + elem.select_remote["name"] + "/" + branch_to_checkout
        debug.verbose("execute : " + cmd)
        ret = multiprocess.run_command(cmd, cwd=git_repo_path)
        if ret[1] != "" and ret != False:
            debug.info("'" + str(ret) + "'")
            debug.error("checkout " + base_display + " ==> Can not checkout to the correct branch")
            return False
        debug.info("checkout " + base_display + " ==> create new branch")
        return True
    # Checkout a specific tags:
    if elem.tag in list_tags:
        debug.info("    ==> find ...")
    else:
        debug.info("checkout " + base_display + " ==> NO remote tags")
        return True
    # checkout the new branch:
    cmd = "git checkout --quiet " + elem.tag + " -b " + branch_to_checkout
    # + " --track " + elem.select_remote["name"] + "/" + branch_to_checkout
    debug.verbose("execute : " + cmd)
    ret = multiprocess.run_command(cmd, cwd=git_repo_path)
    if ret[1] != "" and ret != False:
        debug.info("'" + str(ret) + "'")
        debug.error("checkout " + base_display + " ==> Can not checkout to the correct tags")
        return False
    debug.info("checkout " + base_display + " ==> create new branch: " + branch_to_checkout)
    return True


def get_current_version_repo(git_repo_path):
    version_path_file = os.path.join(git_repo_path, "version.txt")
    add_in_version_management = False
    version_description = None
    if os.path.exists(version_path_file) is False:
        debug.info("deliver:     ==> No 'version.txt' file ==> not manage release version....")
        # Action to do:
        valid = False
        while valid is False:
            debug.info("Create a new version: (0.0.0)")
            debug.info("    (1) Add in managing version")
            debug.info("    (2) Do NOTHING & continue")
            input1 = input()
            if input1 in ["1", "2"]:
                valid = True
            else:
                debug.info("!!! Must select in range " + str(["1", "2"]))
        if input1 == "1":
            version_description = [0, 0, 0]
            add_in_version_management = True
        elif input1 == "2":
            debug.info("Continue Not managing for this repository")
            return (None, None)
        else:
            debug.warning("An error occured for this repository")
            return (None, None)
    else:
        version_description = tools.version_string_to_list(tools.file_read_data(version_path_file))
    return (version_description, add_in_version_management)


def create_new_version_repo(
    git_repo_path,
    version_description,
    add_in_version_management,
    source_branch,
    destination_branch,
):
    # get tracking branch
    ret_destination_branch_sha1 = commands.get_revision_list_to_branch(git_repo_path, destination_branch)
    ret_source_branch_sha1 = commands.get_revision_list_to_branch(git_repo_path, source_branch)
    # remove all identical sha1 ==> not needed for this
    have_forward = False
    for elem_sha1 in ret_destination_branch_sha1:
        if elem_sha1 not in ret_source_branch_sha1:
            message = commands.get_specific_commit_message(git_repo_path, elem_sha1)
            debug.warning("deliver:      Forward commit: '" + message + "'")
            have_forward = True
    if have_forward is True:
        debug.error("'" + destination_branch + "' branch must not be forward '" + source_branch + "' branch")
        return None
    behind_message = ""
    behind_count = 0
    for elem_sha1 in ret_source_branch_sha1:
        if elem_sha1 not in ret_destination_branch_sha1:
            message = commands.get_specific_commit_message(git_repo_path, elem_sha1)
            behind_count += 1
            behind_message = message
    if behind_count == 0 and add_in_version_management is False:
        debug.info("deliver:     ==> Nothing to do (1).")
        return None
    if behind_count == 1 and (behind_message == default_behind_message or behind_message == default_update_message):
        debug.info("deliver:     ==> Nothing to do (2).")
        return None
    for elem_sha1 in ret_source_branch_sha1:
        if elem_sha1 not in ret_destination_branch_sha1:
            message = commands.get_specific_commit_message(git_repo_path, elem_sha1)
            debug.info("deliver:      Behind  commit: '" + message + "'")
    # Choice of the new version:
    valid = False
    while valid is False:
        debug.info("update version: curent: " + str(version_description))
        debug.info("    (1) Major version  (change API)")
        debug.info("    (2) Medium version (add feature)")
        debug.info("    (3) Minor version  (Bug fix & doc)")
        debug.info("    (4) Do not release & continue")
        input1 = input()
        if input1 in ["1", "2", "3", "4"]:
            valid = True
        else:
            debug.info("!!! Must select in range " + str(["1", "2", "3", "4"]))
    # limit and force at 3 the nuber of variables
    version_description_tmp = version_description
    version_description = []
    if len(version_description_tmp) >= 1:
        version_description.append(version_description_tmp[0])
    else:
        version_description.append(0)
    if len(version_description_tmp) >= 2:
        version_description.append(version_description_tmp[1])
    else:
        version_description.append(0)
    if len(version_description_tmp) >= 3:
        version_description.append(version_description_tmp[2])
    else:
        version_description.append(0)
    debug.info("update version: curent: " + str(version_description))
    # increment the version
    if input1 == "1":
        version_description[0] += 1
        version_description[1] = 0
        version_description[2] = 0
    elif input1 == "2":
        version_description[1] += 1
        version_description[2] = 0
    elif input1 == "3":
        version_description[2] += 1
    elif input1 == "4":
        debug.info("No release for this repository")
        return None
    else:
        debug.warning("An error occured for this repository")
        return None
    debug.info("update version: curent: " + str(version_description))
    return version_description


def deliver_push(elem, argument_remote_name, destination_branch, source_branch, base_display):
    # Check the repo exist
    git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
    if os.path.exists(git_repo_path) is False:
        debug.warning("deliver-push: " + base_display + " ==> MUST be download")
        return
    # check if we are on destination_branch
    select_branch = commands.get_current_branch(git_repo_path)
    if select_branch != destination_branch:
        debug.warning("deliver-push: " + base_display + " ==> MUST be on '" + destination_branch + "'")
        return
    # check if we have a local branch
    list_branch_local = commands.get_list_branch_local(git_repo_path)
    if source_branch not in list_branch_local:
        debug.warning("deliver-push: " + base_display + " ==> No '" + source_branch + "' (not managed)")
        return
    if destination_branch not in list_branch_local:
        debug.warning("deliver-push: " + base_display + " ==> No '" + destination_branch + "' (not managed)")
        return
    list_of_element_to_push = []
    # check sha1 of destination_branch
    sha_1_destination = commands.get_sha1_for_branch(git_repo_path, destination_branch)
    tracking_remote_branch = commands.get_tracking_branch(git_repo_path, argument_remote_name, destination_branch)
    if tracking_remote_branch == None:
        debug.warning("deliver-push: " + base_display + " ==> '" + destination_branch + "' have no tracking branch")
        deliver_available = False
    sha_1_destination_tracking = commands.get_sha1_for_branch(git_repo_path, tracking_remote_branch)
    if sha_1_destination == sha_1_destination_tracking:
        debug.info("deliver-push: " + base_display + " ==> '" + destination_branch + "' && '" + tracking_remote_branch + "' have the same sha1")
    else:
        list_of_element_to_push.append(destination_branch)
    # check sha1 of source_branch
    sha_1_source = commands.get_sha1_for_branch(git_repo_path, source_branch)
    tracking_remote_branch = commands.get_tracking_branch(git_repo_path, argument_remote_name, source_branch)
    if tracking_remote_branch == None:
        debug.info("deliver-push: " + base_display + " ==> '" + source_branch + "' have no tracking branch")
        deliver_available = False
    sha_1_source_tracking = commands.get_sha1_for_branch(git_repo_path, tracking_remote_branch)
    if sha_1_source == sha_1_source_tracking:
        debug.info("deliver-push: " + base_display + " ==> '" + source_branch + "' && '" + tracking_remote_branch + "' have the same sha1")
    else:
        list_of_element_to_push.append(source_branch)
    ret_current_tags = commands.get_tags_current(git_repo_path)
    if len(ret_current_tags) == 0:
        debug.info("deliver-push: " + base_display + " ==> No tag on the current '" + destination_branch + "'")
        return
    if len(ret_current_tags) > 1:
        debug.info(
            "deliver-push: "
            + base_display
            + " ==> Too mush tags on the current '"
            + destination_branch
            + "' : "
            + str(ret_current_tags)
            + " ==> only support 1"
        )
        return
    list_remote_tags = commands.get_tags_remote(git_repo_path, argument_remote_name)
    debug.verbose("remote tags: " + str(list_remote_tags))
    if ret_current_tags[0] not in list_remote_tags:
        debug.info("deliver-push: " + base_display + " ==> tag already exist.")
        list_of_element_to_push.append(ret_current_tags[0])
    if len(list_of_element_to_push) == 0:
        debug.info("deliver-push: " + base_display + " ==> Everything up-to-date")
        return
    debug.info("deliver-push: " + base_display + " ==> element to push:" + str(list_of_element_to_push))
    # push all on the server:
    commands.push(git_repo_path, argument_remote_name, list_of_element_to_push)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Repository management.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

from realog import debug

from . import env, tools


class RepoConfig:
    def __init__(self):
        self.name = ""
        self.path = ""
        self.remotes = []  # list of all remotes, with the upstream elements (needed for third party integrations)
        self.select_remote = ""
        self.branch = ""
        self.tag = None
        self.volatile = False


def split_repo(git_repo):
    debug.verbose("parse git repo in RAW: " + str(git_repo))
    if len(git_repo) > 4 and git_repo[:4] == "http":
        # http://wdfqsdfqs@qsdfqsdf/qsdfqsdf/qsdfqsdf/qsdfqs.git find the 3rd '/' and cut at this point
        elements = git_repo.split("/")
        if len(elements) < 4:
            debug.error("Can not parse the git repository : '" + str(git_repo) + "' wrong format http?://xxx@xxx.xxx/****")
        base = elements[0] + "/" + elements[1] + "/" + elements[2]
        repo = git_repo[len(base) + 1 :]
    elif len(git_repo) > 3 and git_repo[:3] == "git":
        # git@qsdfqsdf:qsdfqsdf/qsdfqsdf/qsdfqs.git find the 1st ':' and cut at this point
        elements = git_repo.split(":")
        if len(elements) < 2:
            debug.error("Can not parse the git repository : '" + str(git_repo) + "' wrong format git@xxx.xxx:****")
        base = elements[0]
        repo = git_repo[len(base) + 1 :]
    else:
        debug.error("Can not parse the git repository : '" + str(git_repo) + "' does not start with ['http', 'git']")
    debug.verbose("    base: " + str(base))
    debug.verbose("    repo: " + str(repo))
    return (base, repo)



# -*- coding: utf-8 -*-
"""Action script for manifest-status.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import status

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Display status spécifically of the manifest"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(_my_args, _section):
    _my_args.add(
        "t",
        "tags",
        haveParam=False,
        desc="Display if the commit is on a tag (and display it)",
    )


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_remote_name = ""
    argument_display_tag = False
    for elem in _arguments:
        if elem.get_option_name() == "tags":
            argument_display_tag = True
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()
    elem = configuration.get_manifest_config()
    base_display = tools.get_list_base_display(0, 0, elem)
    ret = status.display_status(elem, argument_remote_name, argument_display_tag, 0, base_display)
    if ret != None:
        return env.ret_action_need_update



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Command interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import errno
import fnmatch
import os
import shutil
import stat
from pathlib import Path

# Local import
from realog import debug

from . import (
    env,
    multiprocess,
)


def check_repository_is_modify(path_repository):
    # check if the repository is modify
    cmd = "git diff --quiet"
    debug.verbose(f"execute : {cmd}")
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value,
        "check_repository_is_modify",
        error_only=True,
        available_return=[0, 1],
        display_if_nothing=False,
    )
    ret_diff = return_value
    if ret_diff[0] == 0:
        return False
    return True


def get_list_branch_meta(path_repository):
    # get local branch
    cmd = "git branch -a"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "get_list_branch_meta", error_only=True
    )
    ret_branch = return_value
    list_branch = ret_branch[1].split("\n")
    out = []
    for elem_branch in list_branch:
        is_remote = False
        branch_name = ""
        is_selected = False
        if len(elem_branch.split(" -> ")) != 1:
            continue
        # separate the remote element
        if elem_branch[2:10] == "remotes/":
            elem_branch = elem_branch[:2] + elem_branch[10:]
            is_remote = True
        # separate select branch
        if elem_branch[:2] == "* ":
            is_selected = True
            branch_name = elem_branch[2:]
        else:
            branch_name = elem_branch[2:]
        out.append({"remote": is_remote, "name": branch_name, "select": is_selected})
    debug.extreme_verbose("List all branch Meta: " + str(out))
    return out


def get_list_branch_all(path_repository):
    tmp = get_list_branch_meta(path_repository)
    out = []
    for elem in tmp:
        out.append(elem["name"])
    debug.verbose("List all branch: " + str(out))
    return out


def get_list_branch_local(path_repository):
    tmp = get_list_branch_meta(path_repository)
    out = []
    for elem in tmp:
        if elem["remote"] is False:
            out.append(elem["name"])
    debug.verbose("List local branch: " + str(out))
    return out


def get_list_branch_remote(path_repository):
    tmp = get_list_branch_meta(path_repository)
    out = []
    for elem in tmp:
        if elem["remote"] is True:
            out.append(elem["name"])
    debug.verbose("List remote branch: " + str(out))
    return out


def get_current_branch(path_repository):
    tmp = get_list_branch_meta(path_repository)
    for elem in tmp:
        if elem["select"] is True:
            debug.verbose("List local branch: " + str(elem["name"]))
            return elem["name"]
    debug.verbose("List local branch: None")
    return None


def get_current_tracking_branch(path_repository):
    # get  tracking branch
    cmd = "git rev-parse --abbrev-ref --symbolic-full-name @{u}"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    if return_value[1] == "@{u}":
        debug.warning("in '" + path_repository + "' no tracking branch is specify")
        return None
    multiprocess.generic_display_error(
        return_value, "get_current_tracking_branch", error_only=True
    )
    return return_value[1]


def get_revision_list_to_branch(path_repository, branch):
    cmd = "git rev-list " + branch
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "get_revision_list_to_branch", error_only=True
    )
    return return_value[1].split("\n")


def get_specific_commit_message(path_repository, sha_1):
    if sha_1 is None or sha_1 == "":
        return ""
    cmd = "git log --format=%B -n 1 " + sha_1
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "get_specific_commit_message", error_only=True
    )
    return return_value[1].split("\n")[0]


def get_sha1_for_branch(path_repository, branch_name):
    if branch_name == None or branch_name == "":
        return None
    cmd = "git rev-parse " + branch_name
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "get_sha1_for_branch", error_only=True
    )
    return return_value[1].split("\n")[0]


def get_tags_current(path_repository):
    cmd = "git tag --points-at"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "get_tags_current", error_only=True
    )
    list_tags = []
    for elem in return_value[1].split("\n"):
        if elem != "":
            list_tags.append(elem)
    return list_tags


def get_tags(path_repository):
    cmd = "git tag"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "get_tags", error_only=True)
    return return_value[1].split("\n")


def get_tags_remote(path_repository, remote_name):
    if remote_name == "" or remote_name == None:
        return get_current_tracking_branch(path_repository)
    cmd = "git ls-remote --tags " + remote_name
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "get_tags_remote", error_only=True)
    list_element = return_value[1].split("\n")
    debug.verbose(" receive: " + str(list_element))
    # 6bc01117e85d00686ae2d423193a161e82df9a44	refs/tags/0.1.0
    # 7ef9caa51cf3744de0f46352e5aa07bd4980fe89	refs/tags/v0.2.0
    # 870e8e039b0a98370a9d23844f0af66824c57a5f	refs/tags/v0.2.0^{}
    # 16707e17e58f16b3409f8c64df7f595ba7dcf499	refs/tags/v0.3.0
    # dfb97c3dfea776e5c4862dc9f60f8c5ad83b55eb	refs/tags/v0.3.0^{}
    out = []
    for elem in list_element:
        cut = elem.split("\t")
        if len(cut) != 2:
            continue
        if cut[1][-3:] == "^{}":
            # specific usage for the annotated commit
            continue
        if cut[1][:10] == "refs/tags/":
            out.append(cut[1][10:])
        else:
            out.append(cut[1])
    return out


def get_tracking_branch(path_repository, remote_name, select_branch):
    # get tracking branch
    if remote_name == "" or remote_name == None:
        return get_current_tracking_branch(path_repository)
    list_branch_remote = get_list_branch_remote(path_repository)
    debug.extreme_verbose(
        "check if exist "
        + remote_name
        + "/"
        + select_branch
        + " in "
        + str(list_branch_remote)
    )
    if remote_name + "/" + select_branch not in list_branch_remote:
        debug.debug("    ==> can not get remote branch")
        return None
    return remote_name + "/" + select_branch


def merge_branch_on_master(
    path_repository, branch_name, merge_force=True, branch_destination="master"
):
    if branch_name == None or branch_name == "":
        raise "Missing branch name"
    cmd = "git merge "
    if merge_force is True:
        cmd += "--no-ff "
    cmd += (
        branch_name
        + " --message \"Merge branch '"
        + branch_name
        + "' into '"
        + branch_destination
        + "'\""
    )
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "merge_branch_on_master", error_only=True
    )
    return return_value


def call_island_release_script(path_repository: str) -> bool:
    cmd = ".island/release.bash"
    file_path = Path(path_repository) / ".island/release.bash"
    if not file_path.exists():
        return False
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command_direct(cmd, cwd=path_repository)
    multiprocess.generic_display_error(
        return_value, "call_island_release_script", error_only=True
    )
    return True


def add_all(path_repository):
    cmd = "git add -u"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "add_all", error_only=True)
    return return_value


def add_file(path_repository, file_path):
    if not file_path:
        raise IOError("Missing file_path name")
    cmd = "git add " + file_path
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "add_file", error_only=True)
    return return_value


def commit_all(path_repository, comment):
    if not comment:
        raise IOError("Missing comment description")
    cmd = 'git commit -a --message "' + comment + '"'
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "commit_all", error_only=True)
    return return_value


def tag(path_repository, tag_name):
    if not tag_name:
        raise IOError("Missing tag name")
    tag_name = tag_name.replace(" ", "_")
    cmd = "git tag " + tag_name + ' --message "[TAG] create tag ' + tag_name + '"'
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "tag", error_only=True)
    return return_value


def checkout(path_repository, branch_name):
    if branch_name == None or branch_name == "":
        raise "Missing branch name"
    cmd = "git checkout " + branch_name
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "checkout", error_only=True)
    return return_value


def reset_hard(path_repository, destination):
    if destination == None or destination == "":
        raise "Missing destination 'sha1' or 'branch name'"
    cmd = "git reset --hard " + destination
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "reset_hard", error_only=True)
    return return_value


def rebase(path_repository, destination):
    if destination == None or destination == "":
        raise "Missing destination 'sha1' or 'branch name'"
    cmd = "git rebase " + destination
    debug.verbose("execute : " + cmd)
    # TODO: check if the command work correctly
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "rebase", error_only=True)
    return return_value


def clone(path_repository, address, branch_name=None, origin=None):
    if address == None or address == "":
        raise "Missing address"
    cmd = "git clone " + address
    if branch_name != None and branch_name == "":
        cmd += " --branch " + branch_name
    if origin != None and origin == "":
        cmd += " --origin " + origin
    if path_repository != None and path_repository != "":
        cmd += " " + path_repository
    debug.verbose("execute : " + cmd)
    if os.path.exists(path_repository) is True:
        debug.warning("Can not clone repository path already exist")
        return False
    return_value = multiprocess.run_command(cmd)
    multiprocess.generic_display_error(return_value, "clone", error_only=True)
    return return_value


def fetch(path_repository, remote_name, prune=True):
    cmd = "git fetch " + remote_name
    if prune is True:
        cmd += " --prune"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "fetch")
    return return_value


def pull(path_repository, remote_name, prune=True):
    if remote_name == None or remote_name == "":
        raise "Missing remote_name"
    cmd = "git pull " + remote_name
    if prune is True:
        cmd += " --prune"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "pull")
    return return_value


def push(path_repository, remote_name, elements):
    if remote_name == None or remote_name == "":
        raise "Missing remote_name"
    if len(elements) == 0:
        raise "No elements to push on server"
    cmd = "git push " + remote_name
    for elem in elements:
        cmd += " " + elem
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "push")
    return return_value


def submodule_sync(path_repository, remote_name):
    cmd = "git submodule sync"
    debug.verbose("execute : " + cmd)
    return_value = multiprocess.run_command(cmd, cwd=path_repository)
    multiprocess.generic_display_error(return_value, "submodule_sync")
    """
	if ret[:31] == "Synchronizing submodule url for":
		#all is good ...
		debug.info("    " + ret)
	elif     ret != "" \
	     and ret != False:
		# all is good, ready to get the system work corectly
		debug.info("'" + ret + "'")
		debug.error("Can not sync submodules ... ")
	"""


def get_forward(path_repository, branch_name):
    if branch_name == None or branch_name == "":
        raise "get_fast_forward: Missing branch_name"
    select_branch = get_current_branch(path_repository)
    # get tracking branch
    ret_current_branch_sha1 = get_revision_list_to_branch(
        path_repository, select_branch
    )
    ret_track_branch_sha1 = get_revision_list_to_branch(path_repository, branch_name)
    # count the number of commit fast forward
    forward_count = 0
    for elem_sha1 in ret_current_branch_sha1:
        if elem_sha1 not in ret_track_branch_sha1:
            forward_count += 1
    return forward_count


def is_forward(path_repository, branch_name):
    return get_forward(path_repository, branch_name) != 0


def get_behind(path_repository, branch_name):
    if branch_name == None or branch_name == "":
        raise "get_fast_forward: Missing branch_name"
    select_branch = get_current_branch(path_repository)
    # get tracking branch
    ret_current_branch_sha1 = get_revision_list_to_branch(
        path_repository, select_branch
    )
    ret_track_branch_sha1 = get_revision_list_to_branch(path_repository, branch_name)
    # count the number of commit behind
    behind_count = 0
    for elem_sha1 in ret_track_branch_sha1:
        if elem_sha1 not in ret_current_branch_sha1:
            behind_count += 1
    return behind_count


def is_behind(path_repository, branch_name):
    return get_behind(path_repository, branch_name) != 0



# -*- coding: utf-8 -*-
"""Action script for volatile-add.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Add a 'volatile' repository with a local path (this element is update as an element in the manifest but is not managed by the manifest)"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    my_args.add_arg("git repository", optionnal=False, desc="Git repositoty to download")
    my_args.add_arg("path", optionnal=False, desc="Path to install the new git repository")


#
# @brief at the end of the help wa have the example section
# @return (string) the Example description string
#
def help_example():
    return "island volatile-add https://git.heeroyui.org/atria-tools/island.git git"


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -5   : env.ret_manifest_is_not_existing      : Manifest does not exit
#         -10  : env.ret_action_is_not_existing        : ACTION is not existing
#         -11  : env.ret_action_executing_system_error : ACTION execution system error
#         -12  : env.ret_action_wrong_parameters       : ACTION Wrong parameters
#         -13  : env.ret_action_partial_done           : ACTION partially done
#
def execute(_arguments):
    if len(_arguments) == 0:
        debug.error("Missing argument to execute the current action [git repository] [path]")

    # the configuration available:
    path = ""
    address_git = ""
    for elem in _arguments:
        if elem.get_option_name() == "git repository":
            address_git = elem.get_arg()
        elif elem.get_option_name() == "path":
            path = elem.get_arg()
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    if address_git == "":
        debug.error(
            "volatile-add: Missing git repository address",
            env.ret_action_wrong_parameters,
        )

    debug.info("Add 'volatile' repository: '" + address_git + "' path='" + path + "'")

    # check system is OK
    manifest.check_island_is_init()

    # Update the current configuration:
    conf = config.get_unique_config()
    # TODO: Check if the local path does not exist in the manifest

    if False == conf.add_volatile(address_git, path):
        return env.ret_action_executing_system_error
    conf.store()
    return None



# -*- coding: utf-8 -*-
"""Action script for init.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Init a island repository (need 'fetch' after)"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    my_args.add("b", "branch", haveParam=True, desc="Select branch to display")
    my_args.add("m", "manifest", haveParam=True, desc="Name of the manifest")


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    if len(_arguments) == 0:
        debug.error("Missing argument to execute the current action ...")

    # the configuration available:
    branch = "master"
    manifest_name = "default.xml"
    address_manifest = ""
    for elem in _arguments:
        if elem.get_option_name() == "branch":
            debug.info("find branch name: '" + elem.get_arg() + "'")
            branch = elem.get_arg()
        elif elem.get_option_name() == "manifest":
            debug.info("find mmanifest name: '" + elem.get_arg() + "'")
            manifest_name = elem.get_arg()
        elif elem.get_option_name() == "":
            if address_manifest != "":
                debug.error("Manifest adress already set : '" + address_manifest + "' !!! '" + elem.get_arg() + "'")
            address_manifest = elem.get_arg()
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    if address_manifest == "":
        debug.error("Init: Missing manifest name")

    debug.info("Init with: '" + address_manifest + "' branch='" + branch + "' name of manifest='" + manifest_name + "'")

    # check if .XXX exist (create it if needed)
    if manifest.is_island_init() is True:
        debug.error("System already init: path already exist: '" + str(env.get_island_path()) + "'")

    tools.create_directory(env.get_island_path())
    # check if the git of the manifest if available

    # create the file configuration:
    conf = config.get_unique_config()
    conf.set_manifest(address_manifest)
    conf.set_branch(branch)
    conf.set_manifest_name(manifest_name)
    conf.store()

    debug.info("Clone the manifest")
    ret_values = commands.clone(env.get_island_path_manifest(), address_manifest, branch_name=branch)

    if ret_values is False:
        debug.info("'" + str(ret_values) + "'")
        debug.error("Init does not work")
        return False

    debug.info("Init done correctly ...")

    return None



# -*- coding: utf-8 -*-
"""Action script for Checkout.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import status

from island import (
    config,
    env,
    manifest,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help() -> str:
    return "Checkout a specific branch in all repository"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the current action
#
def add_specific_arguments(my_args, section):
    my_args.add("r", "remote", haveParam=True, desc="Name of the remote server")
    my_args.add_arg(
        "branch",
        optionnal=False,
        desc="Branch to checkout (if '__TAG__' ==> checkout specific repository tags)",
    )


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_remote_name = ""
    branch_to_checkout = ""
    for elem in _arguments:
        if elem.get_option_name() == "remote":
            debug.info("find remote name: '" + elem.get_arg() + "'")
            argument_remote_name = elem.get_arg()
        elif elem.get_option_name() == "branch":
            branch_to_checkout = elem.get_arg()
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    # update the local configuration file:
    configuration.set_branch(branch_to_checkout)
    configuration.store()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")

    mani = manifest.Manifest(file_source_manifest)

    all_project = mani.get_all_configs()
    debug.info("checkout of: " + str(len(all_project)) + " projects")
    id_element = 0
    have_error = False
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        if status.checkout_elem(elem, argument_remote_name, branch_to_checkout, base_display) is False:
            have_error = True
    if have_error is True:
        return env.ret_action_fail



# -*- coding: utf-8 -*-
"""Action script for sync.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import update_links

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Syncronize all the repository referenced"


#
# @brief at the end of the help wa have the example section
# @return (string) the Example description string
#
def help_example():
    return "island init https://git.heeroyui.org/atria-tools/island.git"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    my_args.add(
        "d",
        "download",
        haveParam=False,
        desc="Just download the 'not download' repository",
    )


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    just_download = False
    for elem in _arguments:
        if elem.get_option_name() == "download":
            just_download = True
            debug.info("find remote name: '" + elem.get_arg() + "'")
        else:
            debug.error("SYNC Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    # TODO: Load Old manifect to check diff ...

    debug.info("update manifest : '" + str(env.get_island_path_manifest()) + "'")
    is_modify_manifest = commands.check_repository_is_modify(env.get_island_path_manifest())
    if is_modify_manifest is True:
        commands.fetch(env.get_island_path_manifest(), "origin")
    else:
        commands.pull(env.get_island_path_manifest(), "origin")

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")

    mani = manifest.Manifest(file_source_manifest)

    all_project = mani.get_all_configs()
    debug.info("synchronize : " + str(len(all_project)) + " projects")
    id_element = 0
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.info("sync : " + base_display)
        tools.wait_for_server_if_needed()
        # debug.debug("elem : " + str(elem))
        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        if elem.tag != None:
            debug.warning("Need to select a specific tag version ... " + elem.tag)
        if os.path.exists(git_repo_path) is False:
            # this is a new clone ==> this is easy ...
            # clone the manifest repository
            address_manifest = ""
            # example git@git.plouf.com:basic_folder
            address_manifest = elem.select_remote["fetch"]
            if elem.select_remote["fetch"][0:4] == "git@" and len(elem.select_remote["fetch"][4:].split(":")) <= 1:
                address_manifest += ":"
            else:
                address_manifest += "/"
            address_manifest += elem.name
            debug.info("clone the repo")
            ret = commands.clone(
                git_repo_path,
                address_manifest,
                branch_name=elem.branch,
                origin=elem.select_remote["name"],
            )
            if ret[0] != "" and ret[0] != False:
                # all is good, ready to get the system work corectly
                debug.info("'" + str(ret) + "'")
                debug.error("Clone repository does not work ... ")
                continue
            # add global mirror list
            for mirror in elem.select_remote["mirror"]:
                debug.verbose("Add global mirror: " + str(mirror))
                cmd = "git remote add " + mirror["name"] + " " + mirror["fetch"]
                if mirror["fetch"][0:4] == "git@":
                    cmd += ":"
                else:
                    cmd += "/"
                cmd += elem.name
                ret = multiprocess.run_command_direct(cmd, cwd=git_repo_path)
                if ret != "" and ret != False:
                    # all is good, ready to get the system work corectly
                    debug.info("'" + str(ret) + "'")
                    debug.warning("Can not add global mirror ... ")
                    continue
                debug.verbose("Add global mirror: " + str(mirror) + " (done)")
            # debug.info("plop " + str(elem.select_remote.keys()))
            # check submodule if requested:
            if elem.select_remote["sync"] is True and os.path.exists(os.path.join(git_repo_path, ".gitmodules")) is True:
                debug.info("    ==> update submodule")
                cmd = "git submodule init"
                ret = multiprocess.run_command_direct(cmd, cwd=git_repo_path)
                if ret != "" and ret != False:
                    # all is good, ready to get the system work corectly
                    debug.info("'" + str(ret) + "'")
                    debug.error("Can not init submodules ... ")
                    continue
                cmd = "git submodule update"
                ret = multiprocess.run_command_direct(cmd, cwd=git_repo_path)
                if ret[:16] == "Submodule path '":
                    # all is good ...
                    debug.info("    " + ret)
                elif ret != "" and ret != False:
                    # all is good, ready to get the system work corectly
                    debug.info("'" + str(ret) + "'")
                    debug.error("Can not init submodules ... ")
                    continue
            continue

        if just_download is True:
            debug.info("SYNC: Already downloaded")
            continue

        if os.path.exists(os.path.join(git_repo_path, ".git")) is False:
            # path already exist but it is not used to as a git repo ==> this is an error
            debug.error("path '" + git_repo_path + "' is already existing but not used for a git repository. Clean it and restart")

        # simply update the repository ...
        debug.verbose("Fetching project: ")

        # get tracking branch
        ret_track = commands.get_current_tracking_branch(git_repo_path)
        is_modify = commands.check_repository_is_modify(git_repo_path)
        select_branch = commands.get_current_branch(git_repo_path)

        if is_modify is True:
            # fetch the repository
            commands.fetch(git_repo_path, elem.select_remote["name"])
            debug.warning("[" + elem.name + "] Not update ==> the repository is modified (just fetch)")
            continue
        commands.pull(git_repo_path, elem.select_remote["name"])

        debug.verbose("select branch = '" + select_branch + "' track: '" + str(ret_track) + "'")
        # check submodule if requested:
        if elem.select_remote["sync"] is True and os.path.exists(os.path.join(git_repo_path, ".gitmodules")) is True:
            debug.info("    ==> sync submodule")
            commands.submodule_sync(git_repo_path, elem.select_remote["name"])

    # Update the links:
    have_error = update_links.update(configuration, mani, "sync-local")
    if have_error is True:
        return -1
    return None



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Link interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""


class LinkConfig:
    def __init__(self) -> None:
        self.source = ""
        self.destination = ""
